@@ -69,10 +69,11 @@ function _base64ToArrayBuffer(base64) {
     return bytes
 }
 
-function fido2SuccessRegistrationCallback(attestation) {
+function fido2SuccessRegistrationCallback(attestation, publicKey) {
     const form = document.getElementById(DJANGO_FIDO_FORM_ID)
     form.client_data.value = _arrayBufferToBase64(attestation.response.clientDataJSON)
     form.attestation.value = _arrayBufferToBase64(attestation.response.attestationObject)
+    form.user_handle.value = _arrayBufferToBase64(publicKey.user.id)
     form.submit()
 }
 
@@ -82,6 +83,13 @@ function fido2SuccessAuthenticationCallback(assertion) {
     form.credential_id.value = _arrayBufferToBase64(assertion.rawId)
     form.authenticator_data.value = _arrayBufferToBase64(assertion.response.authenticatorData)
     form.signature.value = _arrayBufferToBase64(assertion.response.signature)
+    if (form.user_handle) {
+        if (assertion.response.userHandle !== undefined && assertion.response.userHandle !== null) {
+            form.user_handle.value = _arrayBufferToBase64(assertion.response.userHandle)
+        } else {
+            form.user_handle.value = ''
+        }
+    }
     form.submit()
 }
 
@@ -110,16 +118,16 @@ async function sendFido2Request(url, is_registration, credenetials_name, form_da
 
         // Decode credentials
         const decoded_credentials = []
-        for (const credential of publicKey[credenetials_name]){
+        for (const credential of publicKey[credenetials_name]) {
             credential.id = _base64ToArrayBuffer(credential.id)
             decoded_credentials.push(credential)
         }
         publicKey[credenetials_name] = decoded_credentials
 
         try {
-            if (is_registration){
+            if (is_registration) {
                 const result = await navigator.credentials.create({ publicKey })
-                fido2SuccessRegistrationCallback(result)
+                fido2SuccessRegistrationCallback(result, publicKey)
             } else {
                 const result = await navigator.credentials.get({ publicKey })
                 fido2SuccessAuthenticationCallback(result)
@@ -137,14 +145,17 @@ async function sendFido2Request(url, is_registration, credenetials_name, form_da
     }
 }
 
-async function sendFido2RegistrationRequest(url, form_data) {
-    await sendFido2Request(url, true, 'excludeCredentials', form_data)
-}
-
 async function sendFido2AuthenticationRequest(url, form_data) {
     await sendFido2Request(url, false, 'allowCredentials', form_data)
 }
 
+const sendFido2RegistrationRequest = async(e) => {
+    e.preventDefault()
+    const form = document.querySelector(`#${DJANGO_FIDO_FORM_ID}`)
+    const form_data = form.querySelector('[name=user]') ? `user=${form.querySelector('[name=user]').value}` : ''
+    await sendFido2Request(form.dataset.url, true, 'excludeCredentials', form_data)
+}
+
 async function startFido2() {
     const form = document.getElementById(DJANGO_FIDO_FORM_ID)
     if (!form)
@@ -176,16 +187,7 @@ async function startFido2() {
     } else if (form.dataset.mode === FIDO2_REGISTRATION_REQUEST) {
         const submit_button = document.getElementById('submit-button')
         if (submit_button) {
-            submit_button.addEventListener('click', async e => {
-                e.preventDefault()
-
-                let form_data = ''
-                const user_field = form.querySelector('[name=user]')
-                if (user_field)
-                    form_data = `user=${user_field.value}`
-
-                await sendFido2RegistrationRequest(form.dataset.url, form_data)
-            })
+            submit_button.addEventListener('click', sendFido2RegistrationRequest)
         }
     } else {
         addFido2Error(TRANSLATIONS.UKNOWN_FIDO_REQUEST)
@@ -197,4 +199,15 @@ document.addEventListener('DOMContentLoaded', () => {
     startFido2()
 }, false)
 
-export {startFido2, FIDO2_REGISTRATION_REQUEST, FIDO2_AUTHENTICATION_REQUEST, addFido2Error, clearFido2Errors}
+export {
+    FIDO2_REGISTRATION_REQUEST,
+    FIDO2_AUTHENTICATION_REQUEST,
+    DJANGO_FIDO_FORM_ID,
+    _arrayBufferToBase64,
+    _base64ToArrayBuffer,
+    addFido2Error,
+    clearFido2Errors,
+    fido2ErrorResponseCallback,
+    sendFido2RegistrationRequest,
+    startFido2,
+}
